% Kernel learning via semi-definite programming (SDP) in binary case
% Coded by Jianhui Chen (jianhui.chen@asu.edu, Arizona State University)

%                Input Parameters
% "G" : the cell containing centered kernel matrix (trace value = 1) 
% "lambda" : regularization parameter
% "a" : the indicator vector

%               Output parameters
% "theta" : the coefficients of kernel matrices
% "time" : computation time in seconds


function [theta time] = Binary_SDP_Theta(G,lambda,a)

time0 = cputime;

len = length(G);

n = length(a);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%             Compute At,b,c, and solve SDP problem by SeDuMi                    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

At = [];
C = [];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
% 1: Set equality constraint(\theta^T * r = 1), K.f = 1;%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

C = [C;1];
At = [At;  [ 0 ones(1, len) ] ];
K.f = 1;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 2: Set nonnegative constraint(\theta \ge 0), K.l = sigma_length;    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tmp = [ ] ;
tmp = [ zeros(len,1) eye(len) ];
At = [At; -tmp];
C = [ C; zeros(len,1) ];
K.l = len;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3. Set cone constraints as indicated in Eq.(93), K.s ;                  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


F_C = [ ones(n,n),a ; a', 0 ];
C = [C;vec(F_C)];


a_cone = [ ] ;

tmp = zeros(n+1,n+1);
tmp(end,end)=1;
a_cone = [ a_cone vec(tmp)];


for i = 1:len
    tmp = zeros( n + 1, n + 1 );
    tmp(1:n, 1:n) = 1/lambda * G{i};
    a_cone = [ a_cone vec(tmp) ];
end
    

At = [At; - a_cone];
K.s = n+1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 4. Set the objective function for the standard minimization problem;%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tmp = zeros(len+1,1);
tmp(1) = - 1;
b = tmp;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 5. Solve the SDP problem using SeDuMi Package;%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[x, y, info]=sedumi(At,b,C,K);
theta = y(2:end);
time = cputime - time0;
